@@ -11,21 +11,29 @@
 ##' @export
 ##' @aliases gof.phreg 
 ##' @examples
-##' data(TRACE)
+##' library(mets)
+##' data(sTRACE)
 ##' 
-##' m1 <- phreg(Surv(time,status==9)~vf+chf+diabetes,data=TRACE) 
+##' m1 <- phreg(Surv(time,status==9)~vf+chf+diabetes,data=sTRACE) 
 ##' gg <- gof(m1)
+##' gg
 ##' par(mfrow=c(1,3))
 ##' plot(gg)
 ##' 
-##' m1 <- phreg(Surv(time,status==9)~strata(vf)+chf+diabetes,data=TRACE) 
+##' m1 <- phreg(Surv(time,status==9)~strata(vf)+chf+diabetes,data=sTRACE) 
 ##' ## to get Martingale ~ dN based simulations
 ##' gg <- gof(m1)
+##' gg
 ##' 
 ##' ## to get Martingale robust simulations, specify cluster in  call 
-##' m1 <- phreg(Surv(time,status==9)~chf+diabetes+cluster(id),data=TRACE) 
+##' sTRACE$id <- 1:500
+##' m1 <- phreg(Surv(time,status==9)~vf+chf+diabetes+cluster(id),data=sTRACE) 
 ##' gg <- gof(m1)
+##' gg
 ##' 
+##' m1 <- phreg(Surv(time,status==9)~strata(vf)+chf+diabetes+cluster(id),data=sTRACE) 
+##' gg <- gof(m1)
+##' gg
 ##' @export
 gof.phreg  <- function(object,n.sim=1000,silent=1,robust=NULL,...)
 {# {{{
@@ -47,39 +55,34 @@ hatti <- matrix(0,nd,nrow(ii))
 obs <- apply(abs(Ut),2,max)
 
 if (is.null(robust)) 
-     if (!is.null(object$id)) robust <- TRUE else robust <- FALSE
+   if (!is.null(object$call.id)) robust <- TRUE else robust <- FALSE
 
-### cluster call or robust \hat M_i(t) based  
-if (robust) {
-        xx <- object$cox.prep
-        S0i <- rep(0,length(xx$strata))
-        S0i[xx$jumps+ 1] <- 1/object$S0
-        Z <- xx$X
-        ZdN <- U <- E <- matrix(0, nrow(xx$X), object$p)
-        E[xx$jumps + 1, ] <- object$E
-        U[xx$jumps + 1, ] <- object$U
-        cumhaz <- c(cumsumstrata(S0i, xx$strata, xx$nstrata))
-        EdLam0 <- apply(E * S0i, 2, cumsumstrata, xx$strata, xx$nstrata)
-        rr <- c(xx$sign * exp(Z %*% coef(object) + xx$offset))
-        MGt <- U[, drop = FALSE] - (Z * cumhaz - EdLam0) * rr * c(xx$weights)
-        ### also weights 
-        w <- c(xx$weights)
-        nn <- nrow(Z)
+   ### cluster call or robust \hat M_i(t) based  
+if (robust) {# {{{
+   xx <- object$cox.prep
+   Z <- xx$X
+   rrw <- c(xx$sign * exp(Z %*% coef(object) + xx$offset)*xx$weights)
+   UdN <- xx$weights[xx$jumps+1]*object$U 
+   ### also weights 
+   nn <- nrow(Z)
 
-	tt <- system.time(simcox1<- .Call("PropTestCoxClust",MGt,Pt,w*rr,Z,cumhaz,EdLam0,10,obs,nn,xx$id,xx$strata,xx$nstrata,xx$jumps))
+   tt <- system.time(simcox1<- .Call("PropTestCoxClust",UdN,Pt,rrw,xx$X,
+	     object$S0,object$E,
+             10,obs,nn,xx$id,xx$strata,xx$nstrata,object$strata.jumps,xx$jumps))
 
-	prt <- n.sim*tt[3]/(10*60)
-	if (prt>1 & silent==0) cat(paste("Predicted time minutes",signif(prt,2),"\n"))
+   prt <- n.sim*tt[3]/(10*60)
+   if (prt>1 & silent==0) cat(paste("Predicted time minutes",signif(prt,2),"\n"))
+   simcox <- .Call("PropTestCoxClust",UdN,Pt,rrw,xx$X,
+	     object$S0,object$E,
+	     n.sim,obs,nn,xx$id,xx$strata,xx$nstrata,object$strata.jumps,xx$jumps)
+} else {# }}}
+###  or dN_i based  # {{{
+   tt <- system.time(simcox1<-.Call("PropTestCox",U,Pt,10,obs,PACKAGE="mets"))
+   prt <- n.sim*tt[3]/(10*60)
+   if (prt>1 & silent==0) cat(paste("Predicted time minutes",signif(prt,2),"\n"))
+   simcox <-  .Call("PropTestCox",U,Pt,n.sim,obs,PACKAGE="mets")
+}# }}}
 
-	simcox <- .Call("PropTestCoxClust",MGt,Pt,w*rr,Z,cumhaz,EdLam0,n.sim,obs,nn,xx$id,rep(0,nn),1,xx$jumps)
-} else {
-### no  or dN_i based  
-	tt <- system.time(simcox1<-.Call("PropTestCox",U,Pt,10,obs,PACKAGE="mets"))
-	prt <- n.sim*tt[3]/(10*60)
-	if (prt>1 & silent==0) cat(paste("Predicted time minutes",signif(prt,2),"\n"))
-
-	simcox <-  .Call("PropTestCox",U,Pt,n.sim,obs,PACKAGE="mets")
-}
 sup <-  simcox$supUsim
 res <- cbind(obs,simcox$pval)
 colnames(res) <- c("Sup|U(t)|","pval")
@@ -91,7 +94,7 @@ prmatrix(round(res,digits=2))
 }
 
 out <- list(jumptimes=object$jumptimes,supUsim=sup,res=res,supU=obs,
-	    pvals=simcox$pval,score=Ut,simUt=simcox$simUt,type="prop",robust=robust)
+    pvals=simcox$pval,score=Ut,simUt=simcox$simUt,type="prop",robust=robust)
 class(out) <- "gof.phreg"
 return(out)
 }# }}}
@@ -157,9 +160,9 @@ if (is.null(modelmatrix)) stop(" must give matrix for cumulating residuals\n");
 cox1 <- phreg(formula,data,offset=NULL,weights=NULL,Z=modelmatrix,cumhaz=FALSE,...) 
 offsets <- cox1$X %*% cox1$coef
 if (!is.null(offset)) offsets <- offsets*offset
-if (!is.null(cox1$strata)) 
-     coxM <- phreg(cox1$model.frame[,1]~modelmatrix+strata(cox1$strata),data,offset=offsets,weights=weights,no.opt=TRUE,cumhaz=FALSE,...)
-else coxM <- phreg(cox1$model.frame[,1]~modelmatrix,data,offset=offsets,weights=weights,no.opt=TRUE,cumhaz=FALSE,...)
+if (!is.null(cox1$strata.name)) 
+     coxM <- phreg(cox1$model.frame[,1]~modelmatrix+strata(cox1$strata),data,offset=offsets,weights=weights,no.opt=TRUE,cumhaz=FALSE,no.var=1,...)
+else coxM <- phreg(cox1$model.frame[,1]~modelmatrix,data,offset=offsets,weights=weights,no.opt=TRUE,cumhaz=FALSE,no.var=1,...)
 nnames <- colnames(modelmatrix)
 
 Ut <- apply(coxM$U,2,cumsum)
@@ -168,8 +171,6 @@ U <- coxM$U
 Ubeta <- cox1$U
 ii <- -solve(cox1$hessian)
 EE <- .Call("vecMatMat",coxM$E,cox1$E,PACKAGE="mets")$vXZ; 
-###print(dim(EE))
-###print(cox1$ZX)
 Pt <- cox1$ZX - EE
 Pt <- apply(Pt,2,cumsum)
 betaiid <- t(ii %*% t(Ubeta))
@@ -291,54 +292,6 @@ class(out) <- c("gof.phreg")
 return(out)
 }# }}}
 
-###gofZ.phregGam  <- function(formula,data,vars,offset=NULL,weights=NULL,breaks=40,equi=TRUE,
-###			n.sim=1000,silent=1,...)
-###{# {{{
-###
-### res <- matrix(0,length(vars),2)
-### colnames(res) <- c("Sup_z |U(tau,z)|","pval")
-### rownames(res) <- vars
-###
-### i <- 1
-###for (vv in vars) {
-### modelmatrix <- cumContr(data[,vv],breaks=breaks,equi=equi)
-###
-###cox1 <- phreg(formula,data,offset=NULL,weights=NULL,Z=modelmatrix,cumhaz=FALSE,...) 
-###offsets <- cox1$X %*% cox1$coe
-###if (!is.null(offset)) offsets <- offsets*offset
-###
-###if (!is.null(cox1$strata)) 
-###     coxM <- phreg(cox1$model.frame[,1]~modelmatrix+strata(cox1$strata),data,offset=offsets,weights=weights,no.opt=TRUE,cumhaz=FALSE,...)
-###else coxM <- phreg(cox1$model.frame[,1]~modelmatrix,data,offset=offsets,weights=weights,no.opt=TRUE,cumhaz=FALSE,...)
-###nnames <- colnames(modelmatrix)
-###
-###Ut <- apply(coxM$U,2,cumsum)
-###jumptimes <- coxM$jumptimes
-###U <- coxM$U
-###Ubeta <- cox1$U
-###ii <- -solve(cox1$hessian)
-###EE <- .Call("vecMatMat",coxM$E,cox1$E,PACKAGE="mets")$vXZ; 
-###Pt <- cox1$ZX - EE
-###Pt <- apply(Pt,2,cumsum)
-###betaiid <- t(ii %*% t(Ubeta))
-###obs <- apply(abs(Ut),2,max)
-###simcox <-  .Call("ModelMatrixTestCox",U,Pt,betaiid,n.sim,obs,PACKAGE="mets")
-###
-### ## pvals efter z i model.matrix sup_z | M(z,tau) | 
-### Utlast <- max(abs(tail(Ut,1)))
-### maxlast <- apply(abs(simcox$last),1,max)
-### pval.last <- mean(maxlast>=Utlast)
-### res[i,] <- c(Utlast,pval.last)
-### i <- i+1
-###}
-###
-###out <- list(res=res, type="modelmatrix")
-###class(out) <- "gof.phreg"
-###
-###return(out)
-###}# }}}
-###
-
 
 ##' @export
 cumContr <- function(data,breaks=4,probs=NULL,equi=TRUE,na.rm=TRUE,unique.breaks=TRUE,...)
@@ -403,10 +356,10 @@ cumContr <- function(data,breaks=4,probs=NULL,equi=TRUE,na.rm=TRUE,unique.breaks
 ##' @author Thomas Scheike and Klaus K. Holst
 ##' @export
 ##' @examples
-##' data(TRACE)
+##' data(tTRACE)
 ##' 
-##' m1 <- phreg(Surv(time,status==9)~strata(vf)+chf+wmi,data=TRACE) 
-##' m2 <- phreg(Surv(time,status==9)~vf+strata(chf)+wmi,data=TRACE) 
+##' m1 <- phreg(Surv(time,status==9)~strata(vf)+chf+wmi,data=tTRACE) 
+##' m2 <- phreg(Surv(time,status==9)~vf+strata(chf)+wmi,data=tTRACE) 
 ##' par(mfrow=c(2,2))
 ##' 
 ##' gofG.phreg(m1)
@@ -432,9 +385,8 @@ stratnames <- paste(stratn,lstrata,sep=":")
 
 if (is.null(cumhaz)) stop("Must run phreg with cumhaz=TRUE (default)"); 
 if (nstrata==1) stop("Stratified Cox to look at baselines");
-###
 
-if (is.null(x$opt) | is.null(x$coef)) fixbeta<- 1 else fixbeta <- 0
+if ((x$no.opt) | is.null(x$coef)) fixbeta<- 1 else fixbeta <- 0
 
 for (i in 0:(nstrata-2))
 for (j in (i+1):(nstrata-1)) { 
@@ -449,7 +401,6 @@ for (j in (i+1):(nstrata-1)) {
       graphics::title(paste("Stratified baselines for",stratn))
       if ((fixbeta==0 | sim==0) & lm ) 
       graphics::legend("topleft",c("Nonparametric","lm"),lty=1,col=1:2)
-###      graphics::legend("topleft",c("Nonparametric","Stratified-Cox-Sim"),lty=1,col=1:3)
       ab <- lm(cumhazi[,2]~-1+cumhazj[,2])
       if (sim==1 & fixbeta==0) {
              Pt <- DLambeta.t <- apply(x$E/c(x$S0),2,cumsumstrata,strata,nstrata)
@@ -506,7 +457,6 @@ lines(x$jumptimes,x$score[,i],type="s",lwd=1.5)
 	   {
 	    xr <- x$Zres[[i]]
 	    obsz <- c(tail(xr$score,1))
-###	    times <- 1:length(obsz)
 	    times <- xr$xaxs
 	    rsU <- max(max(abs(obsz)),max(abs(xr$simUtlast[1:50,])))
 	    plot(times,obsz,type="l",ylim=c(-rsU,rsU),xlab="",ylab="")
